@@ -15,53 +15,33 @@
   return(A %*% B)
 }
 
-calc_beta_w_cov <- function(object) {
-  beta_cov <- get_beta_cov_any_version(object)
-  beta_w_cov <- attr(beta_cov,"beta_v_cov")
-  invL <- calc_invL(object) ## correlation matrix of ranefs is solve((t(invL)%*%(invL)))
-  # invL must be de facto a block matrix when several ranefs
-  if ( ! is.null(invL)) {
-    pforpv <- ncol(beta_cov)
-    v.range <- pforpv+seq(ncol(invL))
-    beta_w_cov[v.range,] <- t(invL) %*% beta_w_cov[v.range,]
-    beta_w_cov[,v.range] <- beta_w_cov[,v.range] %*% invL
-  }
-  return(beta_w_cov)
-}
-
-calcPredVar <- function(Coldnew,X.pv,newZAC=NULL,newZA,beta_w_cov,
+calcPredVar <- function(ColdnewList,X.pv,newZAC=NULL,newZAlist,beta_w_cov,
                         CnewnewList=NULL,
                         invColdoldList=NULL, ## may be null is no newdata.
                         lambda=NULL, ## may be null if no newdata.
                         logdispObject=NULL, ## should remain NULL is disp not requested
                         covMatrix=FALSE,blockSize=100L) {
   nrX <-  nrow(X.pv)
-  if (is.matrix(Coldnew) || inherits(Coldnew,"Matrix") || inherits(Coldnew,"ff")) {
-    Coldnew <- list(dummyid=Coldnew)
-  } ## if (!is.list(Coldnew)) not valid bc ff objects inherit from list()
-  if (is.matrix(newZA) || inherits(newZA,"Matrix") || inherits(newZA,"ff")) {
-    newZA <- list(dummyid=newZA)
-  }
   if ( ( ! covMatrix ) && nrX > blockSize) {
     slices <- unique(c(seq(0L,nrX,blockSize),nrX))
     sliceVar <- function(it) {
       slice <- (slices[it]+1L):slices[it+1L]
       ## here  the problem is that newZA should map the new response levels 
       ## to the 'new' levels of random effects  
-      nrand <- length(newZA) ## or of any other of the lists of matrices
+      nrand <- length(newZAlist) ## or of any other of the lists of matrices
       requiredLevelsList <- lapply(seq_len(nrand), function(rd) {
-        which(colSums(newZA[[rd]][slice,,drop=FALSE])>0L) })
+        which(colSums(newZAlist[[rd]][slice,,drop=FALSE])>0L) })
       locnewZA <- lapply(seq_len(nrand), function(rd) { 
-        newZA[[rd]][slice,requiredLevelsList[[rd]],drop=FALSE] })
+        newZAlist[[rd]][slice,requiredLevelsList[[rd]],drop=FALSE] })
       # predVar in observed points uses C rather than L hence we need to compute ZA.C in all cases ('newZAC")
-      #  andthen we need newZA and Coldnew
-      locColdnew <- lapply(seq_len(nrand), function(rd) { 
-        locC_on <- Coldnew[[rd]][,requiredLevelsList[[rd]],drop=FALSE]
-        attr(locC_on,"isEachNewInOld") <- attr(Coldnew[[rd]],"isEachNewInOld")[requiredLevelsList[[rd]]] ## for non-spatial effects; (qualifies sub cols of sub Cnewold)
+      #  andthen we need newZA and ColdnewList
+      locColdnewList <- lapply(seq_len(nrand), function(rd) { 
+        locC_on <- ColdnewList[[rd]][,requiredLevelsList[[rd]],drop=FALSE]
+        attr(locC_on,"isEachNewInOld") <- attr(ColdnewList[[rd]],"isEachNewInOld")[requiredLevelsList[[rd]]] ## for non-spatial effects; (qualifies sub cols of sub Cnewold)
         return(locC_on)
       })
       locnewZAClist <- lapply(seq_len(nrand), function(rd) { 
-        terme <- locnewZA[[rd]] %id*id% t(locColdnew[[rd]])[] 
+        terme <- locnewZA[[rd]] %id*id% t(locColdnewList[[rd]])[] 
         as.matrix(terme) ## loses names but they are not useful here 
       })
       if (nrand>1L) {newZAC <- do.call(cbind,locnewZAClist)} else {newZAC <- locnewZAClist[[1]]}
@@ -70,10 +50,10 @@ calcPredVar <- function(Coldnew,X.pv,newZAC=NULL,newZA,beta_w_cov,
         locCnewnewList <- lapply(seq_len(nrand), function(rd) { 
           CnewnewList[[rd]][requiredLevelsList[[rd]],requiredLevelsList[[rd]],drop=FALSE] }) 
       } else locCnewnewList <- NULL
-      calcPredVar(Coldnew=locColdnew, ## needed only if newdata in predict() call
+      calcPredVar(ColdnewList=locColdnewList, ## needed only if newdata in predict() call
                   X.pv=X.pv[slice,,drop=FALSE],## problem is that this creates the apparence of new data end more calculations
                   newZAC=newZAC,
-                  newZA=locnewZA, ## either newZA or newZAC needed even if no newdata in predict() call
+                  newZAlist=locnewZA, ## either newZA or newZAC needed even if no newdata in predict() call
                   beta_w_cov=beta_w_cov,CnewnewList=locCnewnewList,
                   invColdoldList=invColdoldList, ## needed only if newdata in predict() call
                   lambda=lambda,
@@ -84,9 +64,9 @@ calcPredVar <- function(Coldnew,X.pv,newZAC=NULL,newZA,beta_w_cov,
   }
   ############################################################
   if (is.null(newZAC)) {
-    nrand <- length(newZA) ## or of any other of the lists of matrices
+    nrand <- length(newZAlist) ## or of any other of the lists of matrices
     newZAClist <- lapply(seq_len(nrand), function(it) {
-      terme <- newZA[[it]] %id*id% t(Coldnew[[it]])[] ## %id*id% further keeps col names of C if tZA==I
+      terme <- newZAlist[[it]] %id*id% t(ColdnewList[[it]])[] ## %id*id% further keeps col names of C if tZA==I
       as.matrix(terme) ## loses names but they are not useful here 
     })
     if (nrand>1L) {newZAC <- do.call(cbind,newZAClist)} else {newZAC <- newZAClist[[1]]}
@@ -97,33 +77,42 @@ calcPredVar <- function(Coldnew,X.pv,newZAC=NULL,newZA,beta_w_cov,
   calcZWZt_mat_or_diag <- function(Z,W,returnMat) { ## fixefVar or fixefVar + a bit of ranefVar
     if (returnMat) {
       return(Z[] %id*id% W[] %id*id% t(Z)[])
-    } else {
-      rownames(W) <- colnames(W) <- NULL ## inhibits a useles warning from Matrix:: dimNamesCheck 
-      premul <- Z[] %id*id% W[]
-      return(rowSums(suppressMessages(premul * Z[]))) ## suppress message("method with signature...") [found by debug(message)]
+    } else { ## returns only the diagonal
+      if (is.vector(W)) {
+        return(rowSums(Matrix_times_Dvec(Z[],W[]) * Z[]))
+      } else {
+        rownames(W) <- colnames(W) <- NULL ## inhibits a useless warning from Matrix:: dimNamesCheck 
+        premul <- Z[] %id*id% W[]
+        return(rowSums(suppressMessages(premul * Z[]))) ## suppress message("method with signature...") [found by debug(message)]
+      }
     }
   }
   predVar <- calcZWZt_mat_or_diag(newAugX,beta_w_cov,covMatrix)
   ## Second component of predVar:
   # Evar: expect over distrib of (hat(beta),hat(v)) of [variance of Xbeta+Zb given (hat(beta),hat(v))]
-  if (! is.null(CnewnewList) ) {
-    nrand <- length(newZA) ## or of any other of the lists of matrices
+  if ( ! is.null(invColdoldList)) { ## must be equivalent to the presence of newdata
+    nrand <- length(newZAlist) ## or of any other of the lists of matrices
     Evarlist <- lapply(seq_len(nrand), function(it) {
-      isEachNewInOld <- attr(Coldnew[[it]],"isEachNewInOld")
+      isEachNewInOld <- attr(ColdnewList[[it]],"isEachNewInOld")
       if ( ! is.null(isEachNewInOld)) { ## non spatial effect: a vector of booleans indicating whether new is in old (qualifies cols of Cnewold)
         Evar <- Diagonal(x=lambda[it]*as.numeric(! isEachNewInOld))
       } else { ## spatial effect
-        Cno_InvCoo_Con <- t(Coldnew[[it]])[] %id*id% (invColdoldList[[it]])[] %id*id% (Coldnew[[it]])[]
-        Evar <- lambda[it] * (CnewnewList[[it]] - Cno_InvCoo_Con)
+        if (is.null(CnewnewList[[it]])) { ## we compute only the var vector
+          Cno_InvCoo_Con <- colSums(ColdnewList[[it]] * ( (invColdoldList[[it]])[] %id*id% (ColdnewList[[it]])[] ) )
+          Evar <- lambda[it] * (1 - Cno_InvCoo_Con)
+        } else { ## full cov matrix
+          Cno_InvCoo_Con <- t(ColdnewList[[it]])[] %id*id% (invColdoldList[[it]])[] %id*id% (ColdnewList[[it]])[]
+          Evar <- lambda[it] * (CnewnewList[[it]] - Cno_InvCoo_Con)
+        }
       } 
-      terme <- calcZWZt_mat_or_diag(newZA[[it]],Evar,covMatrix)
+      terme <- calcZWZt_mat_or_diag(newZAlist[[it]],Evar,covMatrix)
       if (covMatrix) {
         return(as.matrix(terme))  ## loses names but they are not useful here
       } else return(terme)
     })
     if (nrand>1L) {Evar <- Reduce("+",Evarlist)} else {Evar <- Evarlist[[1]]}
     predVar <- predVar + Evar
-  } 
+  }
   # If components for uncertainty in dispersion params were requested,
   #   logdispObject is not NULL
   # If some components ere computable, $$dwdlogdisp should not be NULL
@@ -150,7 +139,7 @@ calcResidVar <- function(object,newdata=NULL) {
   phi.object <- object$phi.object
   if (is.null(phi_outer <- phi.object$phi_outer)) { ## valid whether newdata are NULL or not:
     glm_phi <- phi.object[["glm_phi"]]
-    if (is.null(glm_phi)) glm_phi <- phi.object$get_glm_phi(object)
+    if (is.null(glm_phi)) glm_phi <- .get_glm_phi(object)
     residVar <- predict(glm_phi, newdata=newdata, type="response")
   } else { ## phi, but not glm_phi
     if (length(phi_outer)==1L) {
@@ -170,7 +159,7 @@ calcNewCorrs <- function(object,locdata,which,
     # (1) code  common to different ranef models 
     olduniqueGeo <- attr(object,"info.uniqueGeo")
     geonames <- colnames(olduniqueGeo)
-    newuniqueGeo <- calcUniqueGeo(data=locdata[,geonames,drop=FALSE]) ## It is essential that it keeps the same order as spMMfactorlist -> ULI -> unique. 
+    newuniqueGeo <- calcUniqueGeo(data=locdata[,geonames,drop=FALSE]) ## It is essential that it keeps the same order as .spMMfactorlist -> ULI -> unique. 
     ### rho only used to compute scaled distances
     rho <- getPar(object$ranFix,"rho")
     if ( ! is.null(rho_mapping <- attr(object,"dist_info")$rho.mapping)
@@ -189,16 +178,7 @@ calcNewCorrs <- function(object,locdata,which,
     ## distance matrix and then call to correl fn:
     # (2) code specific to each ranef model
     if ( ! is.null(spatial.model)) {
-      corr.model <- as.character(spatial.model[[1]])
-      if (corr.model=="AR1") {
-        args <-object$ranFix[which(names(object$ranFix) %in% c("ARphi"))]
-        if (which$no) resu$uuCnewold <- args$ARphi^resu$uuCnewold  
-        if (which$nn) resu$uuCnewnew <- args$ARphi^resu$uuCnewnew  
-      } else {
-        args <-object$ranFix[which(names(object$ranFix) %in% c("nu","Nugget"))] ## so that rho=1 in MaternCorr
-        if (which$no) resu$uuCnewold <- do.call(MaternCorr,args=c(args,list(d=resu$uuCnewold)))  
-        if (which$nn) resu$uuCnewnew <- do.call(MaternCorr,args=c(args,list(d=resu$uuCnewnew)))  
-      }
+      resu <- .calc_corr_from_dist(resu, object, spatial.model, which)
     }
   }
   return(resu)
@@ -221,6 +201,25 @@ makenewname <- function(base,varnames) { ## post CRAN 1.4.1
   fvname
 }
 
+.process_variances <- function(variances) {
+  # $respVar implies components
+  if (is.null(variances$predVar)) variances$predVar <- variances$respVar ## may still be NULL
+  if (is.null(variances$residVar)) variances$residVar <- variances$respVar ## may still be NULL
+  if (is.null(variances$respVar)) variances$respVar <- FALSE 
+  # $predVar implies components
+  if (is.null(variances$linPred)) variances$linPred <- variances$predVar ## may still be NULL
+  if (is.null(variances$disp)) variances$disp <- variances$predVar ## may still be NULL
+  if (is.null(variances$predVar)) variances$predVar <- FALSE 
+  # Do not let any component empty
+  if (is.null(variances$fixefVar)) variances$fixefVar <- FALSE 
+  if (is.null(variances$linPred)) variances$linPred <- FALSE 
+  if (is.null(variances$disp)) variances$disp <- FALSE ## uncertaintly on dispersion parameters
+  if (is.null(variances$residVar)) variances$residVar <- FALSE ## uncertaintly on dispersion parameters
+  if (is.null(variances$cov)) variances$cov <- FALSE
+  ##
+  return(variances)
+}
+
 ## (1) for surface prediction: (developed in InferentialSimulation/InferentialSimulation.R)
 ## (2) But also for generation of fixed effects in simulation of nested-effects models
 predict.HLfit <- function(object, newdata = newX, newX = NULL, re.form = NULL,
@@ -239,130 +238,36 @@ predict.HLfit <- function(object, newdata = newX, newX = NULL, re.form = NULL,
     variances$respVar <- variances$sum
     variances$sum <- NULL
   }
+  if (is.null(object$envir)) object$envir <- list2env(list(), ## back-compatibility fix for old objects
+                                                     parent=environment(HLfit_body))
   ## the final components returned as attributes have names ...Var, other terms should be named differently
   checkIntervals <- (substr(x=intervals, nchar(intervals)-2, nchar(intervals))=="Var")
   if (any(!checkIntervals)) warning("Element(s)",intervals[!checkIntervals],"are suspect, not ending in 'Var'.")
   # possible elements in return value: fixefVar, predVar, residVar, respVar
   variances[intervals] <- TRUE 
-  # $respVar implies components
-  if (is.null(variances$predVar)) variances$predVar <- variances$respVar ## may still be NULL
-  if (is.null(variances$residVar)) variances$residVar <- variances$respVar ## may still be NULL
-  if (is.null(variances$respVar)) variances$respVar <- FALSE 
-  # $predVar implies components
-  if (is.null(variances$linPred)) variances$linPred <- variances$predVar ## may still be NULL
-  if (is.null(variances$disp)) variances$disp <- variances$predVar ## may still be NULL
-  if (is.null(variances$predVar)) variances$predVar <- FALSE 
-  # Do not let any component empty
-  if (is.null(variances$fixefVar)) variances$fixefVar <- FALSE 
-  if (is.null(variances$linPred)) variances$linPred <- FALSE 
-  if (is.null(variances$disp)) variances$disp <- FALSE ## uncertaintly on dispersion parameters
-  if (is.null(variances$residVar)) variances$residVar <- FALSE ## uncertaintly on dispersion parameters
-  if (is.null(variances$cov)) variances$cov <- FALSE
-  ##
-  locform <- attr(object$predictor,"oriFormula")
-  ## possible change of random effect terms
-  if (noReForm(re.form)) {
-    locform <- nobarsMM(locform)  ## ie. (if re.form implies that there is no random effect)
-  } else if (inherits(re.form,"formula")) locform <- re.form
-  # checking variables in the data
-  if (length(locform)==3L) locform <- locform[-2] ## removes RHS for checking  vars on RHS etc
-  allvars <- all.vars(locform) 
-  if (variances$residVar) allvars <- unique(c(allvars,all.vars(attr(object$resid.predictor,"oriFormula")))) ## but the 'newdata' may not contain the resid.predictor vars. 
-  if (is.vector(newdata)) { ## ## less well controlled case, but useful for maximization
-    binding <- binding ## :-) binding must be evaluated before newdata is modified
-    newdata <- data.frame(matrix(newdata,nrow=1))
-    if (length(allvars)==ncol(newdata)) {
-      names(newdata) <- allvars
-    } else {
-      stop(paste("(!) newdata has incorrect length. It should match the following variables:\n",paste(allvars,collapse=" ")))
-    }
-  } 
-  ## it is important that newdata remains NULL if it was so initially because it is tested below. Hence copy in locdata
-  if (is.null(newdata)) {
-    locdata <- object$data[,allvars,drop=FALSE]
-  } else {
-    if( is.matrix(newdata) ) newdata <- as.data.frame(newdata)  
-    # so that matrix 'newdata' arguments can be used as in some other predict methods.
-    locdata <- newdata ## FR->FR [,allvars,drop=FALSE] ?
-    if (any(is.na(locdata))) {
-      stop("NA's in required variables from 'newdata'. Prediction not possible.")
-    }
-  }
-  ## preparation for fixed effects
-  allFrames <- HLframes(formula=locform,data=locdata,fitobject=object)
-  newX.pv <- allFrames$X ## contains columns for the offset and columns for the other variables
-  #needNewEta <- ( ( ! is.null(newdata) ) || variances$linPred || ! is.null(re.form))
-  needNewEta <- ( ( ! is.null(newdata) ) || ! is.null(re.form))
-  ## newetaFix must handle NS's in fixef...
-  # est_and_fix <- names(which(!is.na(object$fixef))) ## estimated + etaFix$beta
-  # validnames <- intersect(colnames(allFrames$X) ,est_and_fix) # newX.pv may  contain names of unestimated coeff for the sam reason asthe original MenanFrames$X...
-  validnames <- colnames(object$X.pv) ## we don't want the etaFix cols (detected by bboptim)
-  if (needNewEta) etaFix <- newetaFix(object,allFrames,validnames=validnames)  ## new fixed effects (or [part of] new linear predictor if re.form)      
-  # old comment: if ( ! is.null(betaFix <- attr(object$predictor,"offsetObj")$betaFix)) { ## suppress betaFix cols so that this is consistent with <object>$X.pv 
-  # new comment: newX.pv must intersect non-NA elements of fixef; see comment and code in newetaFix
-  newX.pv <- newX.pv[,validnames,drop=FALSE] 
-  #} 
-  ## preparation for random effects
-  spatial.terms <- findSpatial(locform) ## list of formula terms
-  spatial.model <- spatial.terms[[1L]] ## one formula term, e.g Matern(1|...)
-  if ( ! is.null(newdata)) {
-    if ( ! is.null(spatial.model)) { 
-      if ((asc <- as.character(spatial.model[[1L]])) %in% c("adjacency","ar1","corrMatrix")) { ## AR1  vs ar1 ?
-        stop(paste("Prediction in 'newdata' not implemented or not possible for models including a", asc,"term."))
-      } 
-    } 
-  }
-  ## matching ranef terms of re.form
-  if (noReForm(re.form)) {
-    nrand <- 0L
-  } else {
-    if (inherits(re.form,"formula")) {
-      ranefs <- parseBars(locform)
-      nrand <- length(ranefs)
-      newinold <- sapply(ranefs, function(v) {which(v==attr(object$ZAlist,"ranefs"))})
-      subZAlist <- object$ZAlist[newinold] ## and reordered
-      sublambda <- object$lambda[newinold]
-    } else {
-      ranefs <- attr(object$ZAlist,"ranefs")
-      nrand <- length(ranefs)
-      newinold <- seq(nrand)
-      subZAlist <- object$ZAlist
-      sublambda <- object$lambda
-    }    
-    spatialOne <- which(ranefs == spatial.model) ## strictly a spatial one, not other correlated ones      
-  }
-  #
-  if (nrand>0L) {
-    FL <- spMMFactorList(locform, allFrames$mf, 0L, drop=TRUE) 
-    newZAlist <- FL$Design ## must be ordered as parseBars result...
-    attr(newZAlist,"ranefs") <- ranefs ## required pour computeZAXlist to match the ranefs of LMatrix
-    if ( is.null(spatial.model)) {
-      newZAClist <- computeZAXlist(XMatrix=NULL,ZAlist=newZAlist) 
-      uuCnewnew <- NULL 
-    } else {
-      oldLMatrix <- attr(object$predictor,"LMatrix") ## may be NULL
-      #which <- list(no=( variances$linPred  || needNewEta || ! is.null(newdata)), 
-      #              nn= variances$linPred && ! is.null(newdata))
-      if (is.null(newdata)) {
-        uuCnewold <- tcrossprodCpp(oldLMatrix) ## as in Corr()
-        uuCnewnew <- NULL
-      } else {
-        which <- list(no= needNewEta, nn= TRUE)
-        uunewCorrs <- calcNewCorrs(object=object,locdata=locdata,
-                                   which=which,
-                                   spatial.model=spatial.model)
-        ## matrices, not list of matrices which are constructed later
-        uuCnewnew <- uunewCorrs$uuCnewnew
-        uuCnewold <- uunewCorrs$uuCnewold
-      }
-      if ( ! is.null(uuCnewold)) attr(uuCnewold,"ranefs") <- attr(oldLMatrix,"ranefs") ## required pour computeZAXlist to match the ranefs of LMatrix
-      newZAClist <- computeZAXlist(XMatrix=uuCnewold,ZAlist=newZAlist) ## (typically some ZA's are unaffected)
-    } 
-  } 
-  #
+  variances <- .process_variances(variances)
+  new_X_ZACblob <- .calc_new_X_ZAC(object=object, newdata=newdata, re.form = re.form,
+                 variances=variances, needNewNew=variances$cov)
+  locdata <- new_X_ZACblob$locdata
+  newX.pv <- new_X_ZACblob$newX.pv
+  newZAlist <- new_X_ZACblob$newZAlist
+  nrand <- length(newZAlist) ## may be reduced if non trivial re.form
+  newZAClist <- new_X_ZACblob$newZAClist
+  if (nrand>1L) {
+    newZAC <- do.call(cbind,newZAClist)
+  } else {newZAC <- newZAClist[[1]]}
+  uuCnewold <- new_X_ZACblob$uuCnewold
+  uuCnewnew <- new_X_ZACblob$uuCnewnew
+  oldnewClist <- new_X_ZACblob$oldnewClist  # cf calcNewCorrs
+  subZAlist <- new_X_ZACblob$subZAlist
+  spatialOne <- new_X_ZACblob$spatialOne
+  spatial.model <- new_X_ZACblob$spatial.model
+  newinold <- new_X_ZACblob$newinold
+  etaFix <- new_X_ZACblob$etaFix
+    #
   ## (1) computes fv (2) compute predVar
   ##### fv
-  if (noReForm(re.form)) {
+  if (.noRanef(re.form)) {
     fv <- object$family$linkinv(etaFix) 
   } else if ( is.null(newdata) && ! inherits(re.form,"formula")) {
     fv <- object$fv ## same length including replicates
@@ -372,14 +277,20 @@ predict.HLfit <- function(object, newdata = newX, newX = NULL, re.form = NULL,
       eta <- etaFix
       newZAlist <- NULL
     } else {
+      ## for random-slope model the original eta's can be recomputed as 
+      # object$X.pv %*% fixef(object) + 
+      #    object$ZAlist[[1]] %*% object$strucList[[1]] %*% object$v_h
+      #
       #### precomputation of coeffs
       ## on the gaussian scale, L.v_ori ~ lam C (lam C + phi I)^{-1}y 
       ## new random autocorr term ~ lam c (lam C + phi I)^{-1}y = c C^{-1} L_ori.v_ori = c [t(L_ori)]^{-1} v_ori
       ## [t(L_ori)]^{-1} v_ori can be computed once for all predictions => 'w_h_coeffs'
-      w_h_coeffs <- object$get_w_h_coeffs(object) ## should work for nonspatial models 
+      if (is.null(w_h_coeffs <- object$envir$w_h_coeffs)) { 
+        object$envir$w_h_coeffs <- w_h_coeffs <- .calc_invL_coeffs(object,object$v_h)
+      }
       old_cum_n_u_h <- attr(object$lambda,"cum_n_u_h")
       lcrandfamfam <- attr(object$`rand.families`,"lcrandfamfam")[newinold]
-      augm_w_h_coeffs <- lapply(seq_len(nrand),function(it) {
+      match_old_new_levels <- function(it) {
         oldu.range <- (old_cum_n_u_h[newinold[it]]+1L):(old_cum_n_u_h[newinold[it]+1L])
         if (it %in%  spatialOne) {     # %in% handles zero-length spatialOne...
           return(w_h_coeffs[oldu.range])          
@@ -387,22 +298,25 @@ predict.HLfit <- function(object, newdata = newX, newX = NULL, re.form = NULL,
           oldlevels <- colnames(subZAlist[[it]])
           newlevels <- colnames(newZAClist[[it]])
           interlevels <- intersect(oldlevels,newlevels)
+          oldpos <- which(oldlevels %in% interlevels) ## positions: handle replicates for random-coef
+          newpos <- which(newlevels %in% interlevels)
           oldv <- w_h_coeffs[oldu.range]
           names(oldv) <- oldlevels
           psi_M <- switch(lcrandfamfam[it], 
-                           gaussian = 0,
-                           gamma = 1, 
-                           beta = 1/2, 
-                           "inverse.gamma" = 1
+                          gaussian = 0,
+                          gamma = 1, 
+                          beta = 1/2, 
+                          "inverse.gamma" = 1
           )
           vpsi_M <- object$rand.families[[newinold[it]]]$linkfun(psi_M) 
           ## since vpsi_M can be non-zero, the expectation of the response can be modified in a re.form model compared to the original
-          newv <- rep(vpsi_M,length(newlevels))
+          newv <- rep(vpsi_M,length(newlevels)) ## fills new levels with psi_M
           names(newv) <- newlevels
-          newv[interlevels] <- oldv[interlevels] 
+          newv[newpos] <- oldv[oldpos] 
           return(newv)
         }
-      })
+      }
+      augm_w_h_coeffs <- lapply(seq_len(nrand),match_old_new_levels)
       if (nrand>1L) {
         ZACw <- lapply(seq_len(nrand),function(it) {
           drop(newZAClist[[it]][] %*% augm_w_h_coeffs[[it]]) ## sapply preforms a cbind if everything is matrix (not Matrix)
@@ -426,9 +340,13 @@ predict.HLfit <- function(object, newdata = newX, newX = NULL, re.form = NULL,
     if (ncol(locdata)>0)  attr(resu,"frame") <- locdata 
   }
   ##### (2) predVar
+  if(variances$predVar && identical(attr(object$ZAlist, "anyRandomSlope"),TRUE)) {
+    warning("This prediction variance computation is not yet implemented for random-coefficient models")
+  }
+  
   if(variances$linPred) {
     beta_cov <- get_beta_cov_any_version(object)
-    beta_w_cov <- object$get_beta_w_cov(object)
+    beta_w_cov <- .get_beta_w_cov(object)
     if (inherits(re.form,"formula")) {
       # identifies an selects columns for the [retained ranefs, which are given by newinold 
       subrange <- unlist(lapply(newinold,function(it) {(old_cum_n_u_h[it]+1L):(old_cum_n_u_h[it+1L])}))
@@ -437,7 +355,7 @@ predict.HLfit <- function(object, newdata = newX, newX = NULL, re.form = NULL,
       beta_w_cov <- beta_w_cov[subrange,subrange]
     }
     if ( ! is.null(newdata)) {
-      invColdoldList <- object$get_invColdoldList(object)
+      invColdoldList <- .get_invColdoldList(object)
       ## list for Cnewnew, which enters in  newZA %*% Cnewnew %*% tnewZA, hence should not represent newZA itself 
       if (nrand>0L) newnewClist <- lapply(seq_len(nrand),function(it) {
         if ( it %in% spatialOne) { ## avoids running the next algo which is slow on large matrices
@@ -448,30 +366,10 @@ predict.HLfit <- function(object, newdata = newX, newX = NULL, re.form = NULL,
         return(Cnn)
       }) ## => completely ignores effects removed in re.form 
     } else {
-      invColdolList <- NULL
+      invColdoldList <- NULL
       newnewClist <- NULL
     }
     if (nrand>0L) {
-      ## list for Coldnew, which enters in ZA %id*id% Coldnew %id*id% tnewZA 
-      oldnewClist <- lapply(seq_len(nrand),function(it) {
-        if (it %in% spatialOne) { ## avoids running the next algo which is slow on large matrices
-          oldnewC <- t(uuCnewold)
-        } else {
-          oldlevels <- colnames(subZAlist[[it]])
-          newlevels <- colnames(newZAlist[[it]])
-          if (identical(oldlevels,newlevels)) {
-            oldnewC <- Diagonal(length(oldlevels)) ## replaces old identityMatrix
-          } else {
-            oldornew <- unique(c(oldlevels,newlevels))
-            oldnewC <- diag(length(oldornew))
-            colnames(oldnewC) <- rownames(oldnewC) <- oldornew
-            oldnewC <- oldnewC[oldlevels,newlevels]
-          }
-          attr(oldnewC,"isEachNewInOld") <- newlevels %in% oldlevels  ## but this attr is unevaluated (-> NULL) for spatial models 
-        }
-        return(oldnewC)
-      })
-      #
       loclist <- list(X.pv=newX.pv,beta_w_cov=beta_w_cov,covMatrix=variances$cov,lambda=object$lambda)
       if (!is.null(uuCnewnew)) {
         uuCnewnewList <- lapply(seq_len(nrand),function(it) {
@@ -483,15 +381,9 @@ predict.HLfit <- function(object, newdata = newX, newX = NULL, re.form = NULL,
       } ## else no loclist$CnewnewList (tested in calcPredVar) 
       #
       if ( ! is.null(newdata)) loclist$invColdoldList <- invColdoldList
-      if (nrand==1L) {
-        loclist$Coldnew <- oldnewClist[[1]]
-        loclist$newZA <- newZAlist[[1]]
-        ## but the code for nrand >1 should work for nrand==1:
-      } else {
-        loclist$Coldnew <- oldnewClist
-        loclist$newZA <- newZAlist
-      }
-      if (variances$disp) loclist$logdispObject <- object$get_logdispObject(object)
+      loclist$ColdnewList <- oldnewClist
+      loclist$newZAlist <- newZAlist
+      if (variances$disp) loclist$logdispObject <- .get_logdispObject(object)
       if (variances$cov) {
         respVar <- as.matrix(do.call(calcPredVar,loclist)) ## matrix, not Matrix (assumed below)
         rownames(respVar) <- colnames(respVar) <- rownames(locdata)
@@ -523,7 +415,10 @@ predict.HLfit <- function(object, newdata = newX, newX = NULL, re.form = NULL,
   if (variances$residVar) {
     pw <- object$prior.weights
     if ( ! (attr(pw,"unique") && pw[1L]==1L)) {
-      warning("Prior weights are not taken in account in residVar computation.")
+      if (! identical(spaMM.getOption("prior_weights_residvar_warned"),TRUE)) {
+        warning("Prior weights are not taken in account in residVar computation.")
+        spaMM.options(prior_weights_residvar_warned=TRUE)
+      }
     }
     if (object$family$family %in% c("poisson","binomial","COMPoisson","negbin")) {
       attr(resu,"residVar") <- object$family$variance(fv)
@@ -648,7 +543,7 @@ print.vcov.HLfit <-function(x, expanded=FALSE, ...) {
 } # Use unlist() to remove attributes from the return value
 
 print.predictions <- function (x, expanded=FALSE, ...) {
-  asvec <- as.vector(x)
+  asvec <- as.vector(x) ## important to remove names and keep them separately
   rnames <- rownames(x)
   toolong <- nchar(rnames)>9
   rnames[toolong] <- paste(substr(rnames[toolong],0,8),".",sep="")
